/* attribute 'sample' - doesn't do anything */
/* $Id: at_sample.h,v 1.1 1993/05/26 23:22:27 coleman Exp $ */

#if !at_sample_h
#define at_sample_h 1

#include <aglo.h>

/* Something like this should be at the top. */

void at_sample(aglo_graph graph);

#endif
